/***************************************************************************/
/* GAUSS Benchmark program version 3.0                                     */
/* Author : Stefan Steinhaus                                               */
/* EMAIL  : stefan@steinhaus-net.de                                        */
/* This program is public domain. Feel free to copy it freely.             */
/***************************************************************************/

proc g1(x,y,z);
	retp(sin(x^2)+sin(y^2)+sin(z^2));
endp;

proc g2(x,y,z);
	retp(2^sin(x)+3^cos(y)+4^sin(z));
endp;

proc g3(x,y,z);
	retp(2^log(x^2+y^2+z^2));
endp;

proc g4(x,y,z);
	retp(sin(x^2-y^2-z^2));
endp;

proc g5(x,y,z);
	retp(cos(x^2+y^2+z^2));
endp;

proc g6(x,y,z);
	retp(sin(x^2+y^2+z^2));
endp;

/* Test of IO function - Reading of data from ASCII file */

proc 1 = IOTestASCII(runs);
  local result,zeit,timing,i;
  local fname,datmat,ColName;
  result=0;
  fname="e:\\mathematik\\ncrunch4\\currency2.txt";
  _dxaschdr=1; _dxprint=0;
  for i (1,runs,1);
    timing=0; zeit=hsec;
    {datmat,ColName}=import(fname,0,0);
    timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Test of IO function - Reading of data over ODBC */

proc 2 = IOTestODBC(runs,datmat);
  local result,zeit,timing,i;
  local fname,ret,ColName,currencies,selstmt,t1,t2,t3,t4;
  currencies="Exchange_Rate_Year,Exchange_Rate_Month,Exchange_Rate_Day,AUD,ATS,BEF,BRL,CAD,CNY,DKK,EUR,FIM,FRF,DEM,GRD,HKD,INR,IEP,ITL,JPY,KRW,MYR,MXP,NLG,NOK,NZD,PTE,SEK,ZAR,SGD,SRI,ESP,CHF,TWD,THB,GBP,VEB";
  result=0;
  selstmt="select "$+currencies$+" from [Currency];";
  fname="e:\\mathematik\\ncrunch4\\database.mdb";
  ret = ODBCAddDataSource("Microsoft Access Driver (*.mdb)","benchmark",fname);
  for i (1,runs,1);
    timing=0; zeit=hsec;
    ODBCOpen(1,"benchmark","","");
    ODBCExecuteDirect(1,selstmt);
    datmat=ODBCReadComplete(1);
    {ColName,t1,t2,t3,t4}=ODBCDescribeCol(1);
    ODBCClose(1);
    timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  ret = ODBCDelDataSource("Microsoft Access Driver (*.mdb)","benchmark");
  retp(result/runs,datmat);
endp;

/* Test of Extraction of submatrices and calculation of descriptive stats */
/* Doing the whole process 100 times do get higher timings                */

proc 1 = ExtractTest(runs,datmat);
  local result,zeit,timing,i,j,k;
  local Year_Data, Limits_Area;
  local Min_Year,Max_Year,Mean_Year,GainLoss_Year;
  result=0;
  Limits_Area={1,261,522,784,1045,1305,1565,1827,2088,2349,2610,2871,3131,3158};
  for i (1,runs,1);
    timing=0; zeit=hsec;
    for k (1,100,1);
      for j (1,13,1);
        Year_Data=datmat[Limits_Area[j]:Limits_Area[j+1]-1,4:37];
        Min_Year=minc(Year_Data);
        Max_Year=maxc(Year_Data);
        Mean_Year=meanc(Year_Data);
        GainLoss_Year=100-(Year_Data[1,.]+0.00000001)./(Year_Data[rows(Year_Data),.]+0.00000001)*100;
      endfor;
    endfor;
    timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Test of 2 loops 5000x5000 */

proc 1 = LoopTest(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    a=1;
    timing=0; 
    zeit=hsec;
    for x (1,5000,1);
        for y (1,5000,1);
            a=a+x+y;
        endfor;
    endfor;
    timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* 3800x3800 normal distributed random matrix^1000 */

proc 1 = CreationMatrix(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; b=abs(rndn(3800,3800)/10);
    zeit=hsec; a=(b+1)^1000; timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Sorting of 3000000 values */

proc 1 = SortFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(3000000,1);
    zeit=hsec; b=sortc(a,1); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* FFT of 1048576 random values (2^20) */

proc 1 = FFTFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(1048576,1);
    zeit=hsec; b=fft(a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Numerical solution of 6 3D integrals */

proc 1 = Integration(runs);
  local a,result,i,xl,yl,zl,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; xl=pi|-pi;yl=xl;zl=xl;
	_intord=2;
    zeit=hsec;
    a=intquad3(&g1,xl,yl,zl); a=intquad3(&g2,xl,yl,zl);
    a=intquad3(&g3,xl,yl,zl); a=intquad3(&g4,xl,yl,zl);
    a=intquad3(&g5,xl,yl,zl); a=intquad3(&g6,xl,yl,zl);
    timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Determinant of a 1000x1000 matrix */

proc 1 = Determinant(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(1000,1000);
    zeit=hsec; b=det(a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Inverse of a 1000x1000 matrix */

proc 1 = InverseFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndu(1000,1000);
    zeit=hsec; b=inv(a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Eigenvalues of a 600x600 matrix */

proc 1 = EigenvaluesFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(600,600);
    zeit=hsec; b=eig(a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Cholesky decomposition of a 1000x1000-matrix */

proc 1 = CholeskyFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=moment(rndn(1000,1000),0);
    zeit=hsec; b=chol(a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* 1000x1000 cross-product matrix */

proc 1 = CrossProductFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(1000,1000);
    zeit=hsec; b=moment(a,0); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Calculation of 1000000 fibonacci numbers */

proc 1 = FibonacciFunc(runs);
  local a,b,phi,result,i,timing,zeit;
  result=0;
  phi = 1.6180339887498949;
  for i (1,runs,1);
    timing=0; a=floor(1000*rndu(1000000,1));
    zeit=hsec; b=(phi^a-(-phi)^(-a))/sqrt(5); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Principal Component Analysis over a 500x500 matrix */

proc 1 = PCAFunc(runs);
  local a,p,v,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    a=rndn(500,500);
    timing=0;
    zeit=hsec;
    {p,v,a}=princomp(a,500);
    timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Gamma function over a 1500x1500 matrix */

proc 1 = GammaFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(1500,1500)^2;
    zeit=hsec; b=gamma(a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Gaussian error function over a 1500x1500 matrix */

proc 1 = GaussianFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(1500,1500)^2;
    zeit=hsec; b=erf(a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;

/* Linear regression over 1000x1000 matrix */

proc 1 = LinearRegressionFunc(runs);
  local a,b,result,i,timing,zeit;
  result=0;
  for i (1,runs,1);
    timing=0; a=rndn(1000,1000); b=seqa(1,1,1000);
    zeit=hsec; b=olsqr(b,a); timing=(hsec-zeit)/100;
    result=result+timing;
  endfor;
  retp(result/runs);
endp;



/* Main program */

cls;
format /RD 8,3;
runs=3;              /* Amount of runs for avg. timing -> suggestion at least 2 */
_GUI=0; datmat=0;
ver=sysstate(1,0);
dlibrary -a odbc;
library odbc;
print; print; print;
print "                  !!! GAUSS - Benchmark program !!!";
print "                  ================================="; print; print;
print "Platform: GAUSS " ftos(ver[1],"%*lf",2,0) "." ftos(ver[2],"%*lf",1,0) "." ftos(ver[3],"%*lf",2,0);
print "Displaying the avg. timings over " runs " runs.";
print; print;
print "Function / timing [sec.]                                           result (sec.)";
print "--------------------------------------------------------------------------------";

{timing}=IOTestASCII(runs);
print /flush "IO test (ASCII file)________________________________________ : " timing;
{timing,datmat}=IOTestODBC(runs,datmat);
print /flush "IO test (ODBC)______________________________________________ : " timing;
{timing}=ExtractTest(runs,datmat);
print /flush "Extraction of submatrices & descriptive statistics__________ : " timing;
{timing}=LoopTest(runs);
print /flush "Test of loops_______________________________________________ : " timing;
{timing}=CreationMatrix(runs);
print /flush "3800x3800 normal distributed random matrix^1000_____________ : " timing;
{timing}=SortFunc(runs);
print /flush "3000000 values sorted ascending_____________________________ : " timing;
{timing}=FFTFunc(runs);
print /flush "FFT over 1048576 values (2^20)______________________________ : " timing;
{timing}=Integration(runs);
print /flush "Triple integration__________________________________________ : " timing;
{timing}=Determinant(runs);
print /flush "Determinant of a 1000x1000 random matrix____________________ : " timing;
{timing}=InverseFunc(runs);
print /flush "Inverse of a 1000x1000 uniform distr. random matrix_________ : " timing;
{timing}=EigenvaluesFunc(runs);
print /flush "Eigenvalues of a normal distr. 600x600 randommatrix_________ : " timing;
{timing}=CholeskyFunc(runs);
print /flush "Cholesky decomposition of a 1000x1000-matrix________________ : " timing; 
{timing}=CrossProductFunc(runs);
print /flush "1000x1000 cross-product matrix______________________________ : " timing;
{timing}=FibonacciFunc(runs);
print /flush "Calculation of 1000000 fibonacci numbers____________________ : " timing;
{timing}=PCAFunc(runs);
print /flush "Principal component analysis over a 500x500 matrix__________ : " timing;
{timing}=GammaFunc(runs);
print /flush "Gamma function over a 1500x1500 matrix______________________ : " timing;
{timing}=GaussianFunc(runs);
print /flush "Gaussian error function over a 1500x1500 matrix_____________ : " timing;
{timing}=LinearRegressionFunc(runs);
print /flush "Linear regression over a 1000x1000 matrix___________________ : " timing;
end;

