%***************************************************************************
%* Matlab Benchmark program version 3.0                                    *
%* Author : Stefan Steinhaus                                               *
%* EMAIL  : stefan@steinhaus-net.de                                        * 
%* This program is public domain. Feel free to copy it freely.             *
%***************************************************************************

clc
disp('The following benchmark program will print the average timings')
disp('to calculate the functions by 3 runs.')
disp(' ')
disp(' ')
disp('!!! MATLAB - Benchmarkprogram !!!')
disp('=================================')
disp(' ')

%* Misc. operation *

result=0; runs=3;
for i=1:runs;
   tic;
   importdata('e:\\mathematik\\ncrunch4\\currency2.txt',' ');
   timing=toc;
   result=result+timing;
end;
result=result/runs;
disp(['IO Reading test (ASCII file)_________________________ : ' num2str(result) ' sec.'])

result=0; runs=1;
for i=1:runs;
   tic;
   conn=database('currency','','');
   curs=exec(conn,'select Exchange_Rate_Year,Exchange_Rate_Month,Exchange_Rate_Day,AUD,ATS,BEF,BRL,CAD,CNY,DKK,EUR,FIM,FRF,DEM,GRD,HKD,INR,IEP,ITL,JPY,KRW,MYR,MXP,NLG,NOK,NZD,PTE,SEK,ZAR,SGD,SRI,ESP,CHF,TWD,THB,GBP,VEB from [currency];');
   curs=fetch(curs,4000);
   colnames=columnnames(curs);
   close(curs);
   close(conn);
   timing=toc;
   result=result+timing;
end;
result=result/runs;
disp(['IO Reading test (ODBC)_______________________________ : ' num2str(result) ' sec.'])

result=0;
Limits_Area=[1,261,522,784,1045,1305,1565,1827,2088,2349,2610,2871,3131,3158];
datmat=zeros(size(curs.data));
datmat(:)=[curs.data{:}];
for i=1:runs;
   tic;
   for k=1:100;
      for j=1:13;
         Year_Data=datmat(Limits_Area(j):Limits_Area(j+1)-1,4:37);
         Min_Year=min(Year_Data);
         Max_Year=max(Year_Data);
         Mean_Year=mean(Year_Data);
         GainLoss_Year=100-(Year_Data(1,:)+0.00000001)./(Year_Data(end,:)+0.00000001)*100;
     end;
   end;
   timing=toc;
   result=result+timing;
end;
result=result/runs;
disp(['Extraction of submatrices & descriptive statistics___ : ' num2str(result) ' sec.'])

result=0; a=1;
for i=1:runs;
   tic;
   for x=1:5000;
       for y=1:5000;
           a=a+x+y;
       end;
   end;
   timing=toc;
   result=result+timing;
end;
result=result/runs;
disp(['Loop testing_________________________________________ : ' num2str(result) ' sec.'])
result=0; a=0;
for i=1:runs
   b=abs(randn(3800,3800)/2);
   tic; 
   a=b.^1000; 
   timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['3800x3800 normal distributed random matrix^1000______ : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=randn(3000000,1);
   tic; b=sort(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['3000000 values sorted ascending______________________ : ' num2str(result) ' sec.'])

%* Analysis *

clear a; clear b; result=0;
for i=1:runs
   a=randn(1048576,1);
   tic; b=fft(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['FFT over 1048576 values (2^20)_______________________ : ' num2str(result) ' sec.'])

clear a; clear b; result=0;
for i=1:runs
   tic;
   Q = triplequad('integrnd1',-pi,pi,-pi,pi,-pi,pi,1.0e-2);
   Q = triplequad('integrnd2',-pi,pi,-pi,pi,-pi,pi,1.0e-2);
   Q = triplequad('integrnd3',-pi,pi,-pi,pi,-pi,pi,1.0e-2);
   Q = triplequad('integrnd4',-pi,pi,-pi,pi,-pi,pi,1.0e-2);
   Q = triplequad('integrnd5',-pi,pi,-pi,pi,-pi,pi,1.0e-2);
   Q = triplequad('integrnd6',-pi,pi,-pi,pi,-pi,pi,1.0e-2);
   timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Triple integration over 6 functions__________________ : ' num2str(result) ' sec.'])

%* Algebra *

clear a; clear b; result=0;
for i=1:runs
   a=rand(1000,1000);
   tic; b=det(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Determinant of a 1000x1000 random matrix_____________ : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=rand(1000,1000);
   tic; b=inv(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Inverse of a 1000x1000 uniform distr. random matrix__ : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=randn(600,600);
   tic; c=eig(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Eigenval. of a normal distr. 600x600 randommatrix____ : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=randn(1000,1000);
   a=a'*a;
   tic; b=chol(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Cholesky decomposition of a 1000x1000-matrix_________ : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=randn(1000,1000);
   tic; b=a'*a; timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['1000x1000 cross-product matrix_______________________ : ' num2str(result) ' sec.'])

%* Number theory *

clear a; clear b;
phi = 1.6180339887498949; result=0;
for i=1:runs;
   a=floor(1000*rand(1000000,1));
   tic;
   b=(phi.^a-(-phi).^(-a))/sqrt(5);
   timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Calculation of 1000000 fibonacci numbers_____________ : ' num2str(result) ' sec.'])

%* Stochastic-statistic *

result=0; a=0; b=0;
for i=1:runs;
    a=randn(500,500);
    tic;
	b=princomp(a);
    timing=toc;
    result=result+timing;
end;
result=result/runs;
disp(['Calc. of the principal components of a 500x500 matrix : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=randn(1500,1500)^2;
   tic; b=gamma(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Gamma function over a 1500x1500 matrix_______________ : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=randn(1500,1500)^2;
   tic; b=erf(a); timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Gaussian error function over a 1500x1500 matrix______ : ' num2str(result) ' sec.'])
clear a; clear b; result=0;
for i=1:runs
   a=randn(1000,1000); b=1:1000;
   tic; b=a\b'; timing=toc;
   result=result+timing;
end
result=result/runs;
disp(['Linear regression over a 1000x1000 matrix____________ : ' num2str(result) ' sec.'])