/*--------------------------------------------------------------------------*/
/* Ox Benchmark program version 3.0 from Stefan Steinhaus                   */
/* EMAIL  : stefan@steinhaus-net.de                                         */ 
/*                                                                          */
/* This program is public domain. Feel free to copy it freely.              */
/*                                                                          */
/*--------------------------------------------------------------------------*/


#include <oxstd.h>

main()
{
	decl cRep = 3;                                     /* number of repeats */
	decl i, k,j,time, a, b, c, x,y,secs,phi,datmat,header;
	decl LimitsArea, YearData, MinYear, MaxYear, MeanYear, GainLossYear;
	print("\n\n\n");
	print("!!! Ox - Benchmark program !!!\n");
	print("==============================\n\n\n");
	print("Displaying the avg. timings over ",cRep," runs.\n\n");
	print("Function / timing                                               result (sec.)\n");
	print("--------------------------------------------------------------------------------\n");
	format("%8.3f");

	/* IO operation */

	for (i = secs = 0; i < cRep; ++i)
	{
		time = timer();
		decl file = fopen("Currency1.txt", "rb");
		fscan(file, "%z", &header);
		fscan(file, "%#m", 3159, 38, &datmat);
	    fclose(file);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;
	print("IO test_____________________________________________________: ", secs, "\n");

	/* Test of Extraction of submatrices and calculation of descriptive stats */
	/* Doing the whole process 100 times do get higher timings                */

	LimitsArea={0,260,521,783,1044,1304,1564,1826,2087,2348,2609,2870,3130,3157};
	for (i=secs=0; i < cRep; ++i)
	{
		time=timer();
	    for (k=0; k<100; ++k)
		{
			for (j=0;j<13; ++j)
			{
				YearData=datmat[LimitsArea[j]:LimitsArea[j+1]-1][4:37];
				MinYear=minc(YearData);
				MaxYear=maxc(YearData);
				MeanYear=meanc(YearData);
				GainLossYear=100-(YearData[0][]+0.00000001)./(YearData[rows(YearData)-1][]+0.00000001)*100;
			}
		}
		secs += (timer() - time) / 100;
	}
	secs /= cRep;
	print("Extraction of submatrices & descriptive statistics_________ : ", secs, "\n");

	/* Misc. operation */

	x=0;y=0;a=1;
	for (i = secs = 0; i < cRep; ++i)
	{
		time = timer();
		for (x = 0; x < 5000; ++x)
		{
			for (y = 0; y < 5000; ++y)
			{
				a = a+x+y;
			}
		}
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Loop test___________________________________________________: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		b=1+fabs(rann(3800,3800)/10);
		time = timer();
		a = b .^ 1000;
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("3800x3800 random matrix .^ 1000_____________________________: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		a = rann(1,3000000);
		time = timer();
		b = sortr(a);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Sorting of 3,000,000 random values__________________________: ", secs, "\n");

	/* Analysis */

	for (i = secs = 0; i < cRep; ++i)
	{
		a = rann(1, 1048576);
		time = timer();
		b = fft(a);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("FFT over 1,048,576 random values____________________________: ", secs, "\n");

	/* Algebra */

	for (i = secs = 0; i < cRep; i++)
	{
		a = rann(1000,1000)/10;
		time = timer();
		b = determinant(a);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Determinant of a 1000x1000 random matrix____________________: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		a = ranu(1000,1000);
		time = timer();
		b = invert(a);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Inverse of a 1000x1000 random matrix________________________: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		a = rann(600,600);
		time = timer();
		eigen(a, &b);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Eigenvalues of a 600x600 random matrix______________________: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		a = rann(1000,1000); a = a'a;
		time = timer();
		b = choleski(a);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Choleski decomposition of a 1000x1000 random matrix_________: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		a = rann(1000,1000);
		time = timer();
		b = a'a;
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Creation of 1000x1000 cross-product matrix__________________: ", secs, "\n");

	/* Number theory */

	phi = 1.6180339887498949;
	for (i = secs = 0; i < cRep; i++)
	{
		a = floor(1000*ranu(500000,1));
		time = timer();
		b = (phi^a-(-phi)^(-a))/sqrt(5);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Calculation of 500000 fibonacci numbers_____________________: ",secs, "\n");


	/* Stochastic-statistic */

	// n! = Gamma(n + 1)
	for (i = secs = 0; i < cRep; ++i)
	{
		b=rann(1500,1500).^2;
		time = timer();
		a = gammafact(b);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Gamma function on a 1500x1500 random matrix_________________: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		a = rann(1500,1500).^2;
		time = timer();
		b = erf(a);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Gaussian error function over a 1500x1500 random matrix______: ", secs, "\n");

	for (i = secs = 0; i < cRep; ++i)
	{
		a = rann(1000,1000); b = range(1,1000);  c = 0;
		time = timer();
		ols2r(b, a, &c);
		secs += (timer() - time) / 100;
	}
	secs /= cRep;  delete a;  delete b;
	print("Linear regression over a 1000x1000 random matrix____________: ", secs, "\n");
}

