(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 4.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     13444,        481]*)
(*NotebookOutlinePosition[     14300,        510]*)
(*  CellTagsIndexPosition[     14256,        506]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "Comparison benchmarktest for number cruncher testreport\n(Version 3)\n",
  StyleBox["created 3rd of March 2002 by Stefan Steinhaus",
    FontSize->12]
}], "Subtitle",
  TextAlignment->Center],

Cell[CellGroupData[{

Cell["Misc. operation", "Section",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["IO Test - Reading of data from ASCII file", "Subsection"],

Cell[BoxData[
    \(\(fn = "\<E:\\mathematik\\ncrunch4\\currency1.txt\>";\)\)], "Input"],

Cell[BoxData[
    \(MyFile[fn_] := 
      Module[{rl, f, r, r2}, 
        rl = ReadList[fn, Word, 
            RecordLists \[Rule] True]; \[IndentingNewLine]f = 
          First[rl]; \[IndentingNewLine]r = Rest[rl]; \[IndentingNewLine]r2 = 
          r /. {"\<.\>" \[Rule] "\<\>", 
              x_String \[RuleDelayed] 
                ToExpression[x]}; \[IndentingNewLine]Join[{f}, 
          r2]]\)], "Input"],

Cell[BoxData[
    \(Timing[\(data = MyFile[fn];\)]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["IO Test - Reading of data from ODBC data source", "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(Needs["\<DatabaseAccess`\>"]\)], "Input"],

Cell[BoxData[
    \(Timing[
      Module[{}, dbselect\  = \ OpenDataSource["\<currency\>"]; 
        datmat = 
          DataSourceEvaluate[dbselect, 
            SQLSelect[
              SQLTable["\<[currency]\>"], \ \
{SQLColumn["\<Exchange_Rate_Year\>"], \ 
                SQLColumn["\<Exchange_Rate_Month\>"], \ 
                SQLColumn["\<Exchange_Rate_Day\>"], SQLColumn["\<AUD\>"], 
                SQLColumn["\<ATS\>"], SQLColumn["\<BEF\>"], 
                SQLColumn["\<BRL\>"], SQLColumn["\<CAD\>"], 
                SQLColumn["\<CNY\>"], SQLColumn["\<DKK\>"], 
                SQLColumn["\<EUR\>"], SQLColumn["\<FIM\>"], 
                SQLColumn["\<FRF\>"], SQLColumn["\<DEM\>"], 
                SQLColumn["\<GRD\>"], SQLColumn["\<HKD\>"], 
                SQLColumn["\<INR\>"], SQLColumn["\<IEP\>"], 
                SQLColumn["\<ITL\>"], SQLColumn["\<JPY\>"], 
                SQLColumn["\<KRW\>"], SQLColumn["\<MYR\>"], 
                SQLColumn["\<MXP\>"], SQLColumn["\<NLG\>"], 
                SQLColumn["\<NOK\>"], SQLColumn["\<NZD\>"], 
                SQLColumn["\<PTE\>"], SQLColumn["\<SEK\>"], 
                SQLColumn["\<ZAR\>"], SQLColumn["\<SGD\>"], 
                SQLColumn["\<SRI\>"], SQLColumn["\<ESP\>"], 
                SQLColumn["\<CHF\>"], SQLColumn["\<TWD\>"], 
                SQLColumn["\<THB\>"], SQLColumn["\<GBP\>"], 
                SQLColumn["\<VEB\>"]}]]; 
        colname = 
          DataSourceColumns[dbselect, \ TableName\  -> \ "\<currency\>"]; 
        CloseDataSource[dbselect];]]\)], "Input"],

Cell[BoxData[
    \(Remove[DatabaseAccess]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell["\<\
Data Extraction Test - Extraction of data and descriptive statistics\
\>", "Subsection"],

Cell[BoxData[{
    \(<< Statistics`DescriptiveStatistics`\), "\n", 
    \(\(LimitsArea = {1, 261, 522, 784, 1045, 1305, 1565, 1827, 2088, 2349, 
          2610, 2871, 3131, 3159};\)\), "\n", 
    \(Timing[
      Do[Do[YearData = 
            Take[datmat, {LimitsArea[\([j]\)], 
                LimitsArea[\([j + 1]\)] - 1}, {1, 
                37}]; \[IndentingNewLine]YDcols = 
            Transpose[YearData]; \[IndentingNewLine]MinYear = 
            Map[Min, YDcols]; \[IndentingNewLine]MaxYear = 
            Map[Max, YDcols]; \[IndentingNewLine]MeanYear = 
            Map[Mean, YDcols]; \[IndentingNewLine]GainLossYear = 
            100 - \((First[YearData] + 0.00000001)\)/\((Last[YearData] + 
                      0.00000001)\)*100, {j, 1, 13}], {k, 1, 
          100}]]\)}], "Input"],

Cell[CellGroupData[{

Cell["Loop 5000x5000", "Subsection",
  Evaluatable->False],

Cell[BoxData[{
    \(\(a = 1;\)\), "\[IndentingNewLine]", 
    \(Timing[
      Do[Do[a = a + i + j, {i, 1, 5000, 1}], {j, 1, 5000, 1}]]\)}], "Input",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontSize->12,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3800x3800 random matrix ^ 1000", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real]\/10, {3800}, {3800}];\)\)], "Input",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontSize->12,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Timing[\(\((a + 1)\)\^1000;\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]],

Cell[CellGroupData[{

Cell["3000000 values sorted in ascending order", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real], {3000000}];\)\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(Timing[\(Sort[a];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Analysis", "Section",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["FFT over 1048576 random values", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real], {1048576}];\)\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(Timing[\(Fourier[a];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]],

Cell[CellGroupData[{

Cell["Triple integration", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(\[DoubledPi] = N[\[Pi]];\)\)], "Input"],

Cell[BoxData[
    \(\(SetOpts[NIntegrate, o__?OptionQ] := \((oni = Options[NIntegrate]; 
          SetOptions[NIntegrate, o])\);\)\)], "Input"],

Cell[BoxData[
    \(\(ResetOpts[NIntegrate] := 
        SetOptions[NIntegrate, Sequence @@ oni];\)\)], "Input"],

Cell[BoxData[{
    \(\(SetOpts[NIntegrate, PrecisionGoal \[Rule] 2];\)\), "\n", 
    \(\(t0 = SessionTime[];\)\), "\[IndentingNewLine]", 
    \(\(NIntegrate[
        Sin[x\^2] + Sin[y\^2] + 
          Sin[z\^2], {x, \(-\[DoubledPi]\), \[DoubledPi]}, {y, \
\(-\[DoubledPi]\), \[DoubledPi]}, {z, \(-\[DoubledPi]\), \[DoubledPi]}];\)\), \
"\[IndentingNewLine]", 
    \(\(NIntegrate[
        2\^Sin[x] + 3\^Cos[y] + 
          4\^Sin[z], {x, \(-\[DoubledPi]\), \[DoubledPi]}, {y, \
\(-\[DoubledPi]\), \[DoubledPi]}, {z, \(-\[DoubledPi]\), \[DoubledPi]}];\)\), \
"\n", 
    \(\(NIntegrate[
        2\^Log[x\^2 + y\^2 + z\^2], {x, \(-\[DoubledPi]\), \[DoubledPi]}, {y, \
\(-\[DoubledPi]\), \[DoubledPi]}, {z, 0, \[DoubledPi]}];\)\), "\n", 
    \(\(NIntegrate[
        Sin[x\^2 - y\^2 - 
            z\^2], {x, \(-\[DoubledPi]\), \[DoubledPi]}, {y, \
\(-\[DoubledPi]\), \[DoubledPi]}, {z, \(-\[DoubledPi]\), \[DoubledPi]}];\)\), \
"\n", 
    \(\(NIntegrate[
        Cos[x\^2 + y\^2 + 
            z\^2], {x, \(-\[DoubledPi]\), \[DoubledPi]}, {y, \
\(-\[DoubledPi]\), \[DoubledPi]}, {z, \(-\[DoubledPi]\), \[DoubledPi]}];\)\), \
"\n", 
    \(\(NIntegrate[
        Sin[x\^2 + y\^2 + 
            z\^2], {x, \(-\[DoubledPi]\), \[DoubledPi]}, {y, \
\(-\[DoubledPi]\), \[DoubledPi]}, {z, \(-\[DoubledPi]\), \[DoubledPi]}];\)\), \
"\n", 
    \(SessionTime[] - t0\), "\[IndentingNewLine]", 
    \(\(ResetOpts[NIntegrate];\)\)}], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Algebra", "Section",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Determinant of a 1000x1000 random matrix", "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(a = Table[Random[Real], {1000}, {1000}];\)\)], "Input",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontSize->12,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Timing[\(Det[a];\)]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Inverse of a 1000x1000 random matrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real], {1000}, {1000}];\)\)], "Input",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontSize->12,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Timing[\(Inverse[a];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]],

Cell[CellGroupData[{

Cell["Eigenvalues of a 600x600 randommatrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real], {600}, {600}];\)\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(Timing[\(Eigenvalues[a];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cholesky decomposition of a 1000x1000-matrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(<< "\<LinearAlgebra`Cholesky`\>"\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(\(a = Table[Random[Real], {1000}, {1000}];\)\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(\(a\  += \ Transpose[a]\  + \ 100. \ IdentityMatrix[1000];\)\)], "Input"],

Cell[BoxData[
    \(Timing[\(CholeskyDecomposition[a];\)]\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(Remove[LinearAlgebra`Cholesky]\)], "Input",
  PageWidth->Infinity]
}, Open  ]],

Cell[CellGroupData[{

Cell["1000x1000 cross-product matrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(Timing[\(Transpose[a] . a;\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Number theory", "Section",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Calculation of 1000000 fibonacci numbers", "Subsection",
  Evaluatable->False],

Cell[BoxData[{
    \(gr = N[GoldenRatio]; w5 = N[\@5];\), "\n", 
    \(\(fiboN[n_] = \((gr\^n - \((\(-gr\))\)\^\(-n\))\)/w5;\)\)}], "Input"],

Cell[BoxData[
    \(SeedRandom[16]; 
    a = Table[Random[Integer, {100, 1000}], {1000000}];\)], "Input"],

Cell[BoxData[
    \(Timing[\(fiboN[a];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Stochastic & statistic", "Section",
  CellFrame->True,
  Background->GrayLevel[0.849989]],

Cell[CellGroupData[{

Cell["Principal component analysis over a 500x500 matrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(<< Statistics`MultiDescriptiveStatistics`\)], "Input"],

Cell[BoxData[
    \(\(a = Table[Random[Real], {500}, {500}];\)\)], "Input",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontSize->12,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[\(PrincipalComponents[a];\)]\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(Remove[Statistics`MultiDescriptiveStatistics]\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gamma function over a 1500x1500 matrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real], {1500}, {1500}];\)\)], "Input",
  PageWidth->Infinity,
  FontFamily->"Courier New",
  FontSize->12,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell[BoxData[
    \(Timing[\(Gamma[a];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gaussian error function over a 1500x1500 matrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real], {1500}, {1500}];\)\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(Timing[\(Erf[a];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]],

Cell[CellGroupData[{

Cell["Linear regression over a 1000x1000 matrix", "Subsection",
  Evaluatable->False],

Cell[BoxData[
    \(\(a = Table[Random[Real], {1000}, {1000}];\)\)], "Input",
  PageWidth->Infinity],

Cell[BoxData[
    \(Timing[\(Table[
          Fit[a\[LeftDoubleBracket]i\[RightDoubleBracket], {1, x}, x], {i, 1, 
            1000}];\)]\)], "Input",
  PageWidth->Infinity]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.2 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 951}},
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{801, 873},
WindowMargins->{{1, Automatic}, {Automatic, 0}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 210, 5, 110, "Subtitle"],

Cell[CellGroupData[{
Cell[1989, 60, 88, 2, 75, "Section"],

Cell[CellGroupData[{
Cell[2102, 66, 63, 0, 47, "Subsection"],
Cell[2168, 68, 88, 1, 30, "Input"],
Cell[2259, 71, 412, 9, 110, "Input"],
Cell[2674, 82, 63, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2774, 88, 91, 1, 47, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2890, 93, 61, 1, 30, "Input"],
Cell[2954, 96, 1554, 29, 270, "Input"],
Cell[4511, 127, 55, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],
Cell[4593, 132, 98, 2, 47, "Subsection"],
Cell[4694, 136, 796, 15, 190, "Input"],

Cell[CellGroupData[{
Cell[5515, 155, 58, 1, 47, "Subsection",
  Evaluatable->False],
Cell[5576, 158, 294, 9, 50, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5907, 172, 74, 1, 47, "Subsection",
  Evaluatable->False],
Cell[5984, 175, 226, 7, 42, "Input"],
Cell[6213, 184, 86, 2, 31, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6336, 191, 84, 1, 47, "Subsection",
  Evaluatable->False],
Cell[6423, 194, 95, 2, 30, "Input"],
Cell[6521, 198, 76, 2, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6646, 206, 81, 2, 75, "Section"],

Cell[CellGroupData[{
Cell[6752, 212, 74, 1, 47, "Subsection",
  Evaluatable->False],
Cell[6829, 215, 95, 2, 30, "Input"],
Cell[6927, 219, 79, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7043, 226, 62, 1, 47, "Subsection",
  Evaluatable->False],
Cell[7108, 229, 61, 1, 30, "Input"],
Cell[7172, 232, 143, 2, 30, "Input"],
Cell[7318, 236, 111, 2, 30, "Input"],
Cell[7432, 240, 1423, 32, 220, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8904, 278, 80, 2, 75, "Section"],

Cell[CellGroupData[{
Cell[9009, 284, 84, 1, 47, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[9118, 289, 222, 7, 30, "Input"],
Cell[9343, 298, 52, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9444, 305, 80, 1, 47, "Subsection",
  Evaluatable->False],
Cell[9527, 308, 222, 7, 30, "Input"],
Cell[9752, 317, 79, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9868, 324, 81, 1, 47, "Subsection",
  Evaluatable->False],
Cell[9952, 327, 98, 2, 30, "Input"],
Cell[10053, 331, 83, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10173, 338, 88, 1, 47, "Subsection",
  Evaluatable->False],
Cell[10264, 341, 88, 2, 30, "Input"],
Cell[10355, 345, 100, 2, 30, "Input"],
Cell[10458, 349, 93, 1, 30, "Input"],
Cell[10554, 352, 93, 2, 30, "Input"],
Cell[10650, 356, 86, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10773, 363, 74, 1, 47, "Subsection",
  Evaluatable->False],
Cell[10850, 366, 85, 2, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10984, 374, 86, 2, 75, "Section"],

Cell[CellGroupData[{
Cell[11095, 380, 84, 1, 47, "Subsection",
  Evaluatable->False],
Cell[11182, 383, 140, 2, 53, "Input"],
Cell[11325, 387, 105, 2, 30, "Input"],
Cell[11433, 391, 77, 2, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11559, 399, 95, 2, 75, "Section"],

Cell[CellGroupData[{
Cell[11679, 405, 94, 1, 47, "Subsection",
  Evaluatable->False],
Cell[11776, 408, 74, 1, 30, "Input"],
Cell[11853, 411, 220, 7, 30, "Input"],

Cell[CellGroupData[{
Cell[12098, 422, 91, 2, 30, "Input"],
Cell[12192, 426, 78, 1, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[12319, 433, 82, 1, 47, "Subsection",
  Evaluatable->False],
Cell[12404, 436, 222, 7, 30, "Input"],
Cell[12629, 445, 77, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12743, 452, 91, 1, 47, "Subsection",
  Evaluatable->False],
Cell[12837, 455, 100, 2, 30, "Input"],
Cell[12940, 459, 75, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13052, 466, 85, 1, 47, "Subsection",
  Evaluatable->False],
Cell[13140, 469, 100, 2, 30, "Input"],
Cell[13243, 473, 173, 4, 30, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

